/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#include "stdafx.h"
#include "DumbLuck.h"

// example fitter plugin
// does "tries" iterations,
// each iteration multiplies each curve param by 1.01

QUB_DumbLuck::QUB_DumbLuck()
: state( QUB_Tree::Create("Fitter") ),
// initialize params:
  tries( 5 )
{
	// setup state tree, used for presets
	// Name is required
	state["Name"].setData( getName() );
	// your params here:
	state["tries"].setData( QTR_TYPE_INT, tries );
}
void __cdecl QUB_DumbLuck::Free()
// required for Delphi compatibility
{
	delete this;
}
IQUB_Evts * __cdecl QUB_DumbLuck::Events()
{
	return &events;
}
const char * __cdecl QUB_DumbLuck::getName()
// as displayed in the menu of fitters
{
	return "Dumb-Luck";
}
int __cdecl QUB_DumbLuck::getParamCount()
// your param count here:
{
	return 1;
}
const char * __cdecl QUB_DumbLuck::getParamName(int ix)
// your params here:
{
	if ( ix == 0 )
		return "tries";
	return "";
}
double __cdecl QUB_DumbLuck::getParamVal(int ix)
{
// your params here:
	if ( ix == 0 )
		return tries;
	return 0.0;
}
void __cdecl QUB_DumbLuck::setParamVal(int ix, double val)
// update param and state tree, for presets,
// and notify GUI it was changed
// ignore unreasonable values
{
	if ( ix == 0 ) {
		if ( val >= 1 ) {
			tries = (int) val;
			state["tries"].setData(QTR_TYPE_INT, tries);
			events.send("Fitter ParamVal", this);
		}
	}
}
int __cdecl QUB_DumbLuck::Fit(IQUB_Curve* curve, QF_Data **adata, int ndata)
{
	this->curve = curve;
	this->data = adata[0];
	this->ff.resize(data->n);
	this->iter = 0;
	this->msg = "";
	this->stopRequested = false;

	for (int i=0; (i<tries) && (! stopRequested); ++i) {
		// iteration loop:

		// set new param values in the curve object
		for (int j=curve->getParamCount()-1; j>=0; --j) {
			curve->setParamVal(j, 1.01*curve->getParamVal(j), (j==0)?1:0);
		}
		// use curve object to calculate new curve for fitting procedure
		curve->evalArrayAll(data, &(ff[0]));

		// notify GUI each iteration
		events.send("Fitter Iter", this);
	}

	this->msg = "done";                // send the GUI a message to print out anytime
	events.send("Fitter Msg", this);   // "Fitter Err" would bring up an error dialog

	this->stopRequested = false;
	return 0;
}
void __cdecl QUB_DumbLuck::Stop()
{
	stopRequested = true;
}
int __cdecl QUB_DumbLuck::getIterations()
// update iter in your fitting routine
{
	return iter;
}
double __cdecl QUB_DumbLuck::getParamErr(int ix)
// maybe your procedure gives its own error estimates e.g. from the Hessian matrix.
// otherwise, you can use our rough estimator:
{
	return ChiSqrUncertainty(curve, data, &(ff[0]), ix, 0.01);
}
double * __cdecl QUB_DumbLuck::getCurve()
// the latest fit curve, as calculated during iteration
{
	return &(ff[0]);
}
const char * __cdecl QUB_DumbLuck::getMessage()
{
	return msg.c_str();
}
QTR_Impl* __cdecl QUB_DumbLuck::getState()
// for presets
{
	QTR_Impl* pstate = state.getImpl();
	QTR_INCREF(pstate);
	return pstate;
}
void __cdecl QUB_DumbLuck::updateState(QTR_Impl* pustate)
// for presets
{
	QUB_Tree ustate(pustate);
	if ( ustate["Name"].dataAsString() != this->getName() )
		return;
	// your params here:
	QUB_Tree upar = * ustate.find("tries");
	if ( upar.dataCount() > 0 )
		this->setParamVal(0, upar.dataAsInt());
}

// make your own factory function with this signature,
// and register it in qubfits plugin.cpp
IQUB_Fitter* __stdcall CreateDumbLuck()
{
	return new QUB_DumbLuck;
}


